%% Closed Curves
% This file generates plots to demonstrate the notation for a simple
% closed curve Gamma and its mapping G(Gamma).

%% Transfer Function
b0 = 2;
a0 = 4;
G = tf(b0,[1 a0]);

%% Closed Curves, Gamma
% The curve Gamma goes from s=-jR to s=+jR along the imaginary axis and 
% then returns to s=-jR along a semicircle of radius R in the RHP.

Npts = 500;

R = 50;
Gamma1 = j*linspace(-R,0,Npts);
Gamma2 = j*linspace(0,R,Npts);

theta = linspace(pi/2,-pi/2,Npts);
Gamma3 = R*exp(j*theta);

Gamma = [Gamma1(:); Gamma2(:); Gamma3(:)];

figure(1);
clf
plot(real(Gamma1),imag(Gamma1),'r:', real(Gamma2),imag(Gamma2),'b',...
        real(Gamma3), imag(Gamma3),'g-.');
xlabel('Real Part');
ylabel('Imag. Part');
axis equal
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Label specific points and plot arrowheads 
hold on;
th1 = [];

plot(0,-R,'r.','MarkerSize',30);
th1(1) = text(-21,-0.92*R,'s_1 = -jR');
arrowhead(0,-R/2,0,0,10,[],'r');

plot(0,0,'b.','MarkerSize',30);
th1(2) = text(-18,0,'s_2 = 0');
arrowhead(0,R/2,0,R,10,[],'b');

plot(0,+R,'b.','MarkerSize',30);
th1(3) = text(-21,0.92*R,'s_3 = +jR');
arrowhead(R*cos(pi/4),R*sin(pi/4),...
    R*cos(pi/4-0.01),R*sin(pi/4-0.01),10,[],'g');

plot(+R,0,'g.','MarkerSize',30);
th1(4) = text(R+3,0,'s_4 = +R');
arrowhead(R*cos(-pi/4),R*sin(-pi/4),...
    R*cos(-pi/4-0.01),R*sin(-pi/4-0.01),10,[],'g');

hold off;

%% Mapping, G(Gamma)

G1 = freqresp(G,Gamma1);
G1 = G1(:);
G2 = freqresp(G,Gamma2);
G2 = G2(:);
G3 = freqresp(G,Gamma3);
G3 = G3(:);

figure(2);
clf
plot(real(G1),imag(G1),'r:', real(G2),imag(G2),'b',...
        real(G3), imag(G3),'g-.');
xlabel('Real Part');
ylabel('Imag. Part');
axis equal
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Label specific points and plot arrowheads 
hold on;
th2 = [];

Gs1 = evalfr(G,-j*R);
plot(real(Gs1),imag(Gs1),'r.','MarkerSize',30);
th2(1) = text(0.02,0.06,'G(s_1) = 0.003 + 0.040j');
arrowhead(0.25,0.25,0.26,0.25,0.05,[],'r');

Gs2 = evalfr(G,0);
plot(real(Gs2),imag(Gs2),'b.','MarkerSize',30);
th2(2) = text(0.35,0,'G(s_2) = 0.5');
arrowhead(0.25,-0.25,0.24,-0.25,0.05,[],'b');

Gs3 = evalfr(G,j*R);
Gs4 = evalfr(G,R);
plot(real(Gs3),imag(Gs3),'b.','MarkerSize',30);
th2(3) = text(0.02,-0.06,'G(s_3) = 0.003 - 0.040j');
arrowhead(Gs4*cos(-pi/4),Gs4*sin(-pi/4),...
    Gs4*cos(-pi/4+0.01),Gs4*sin(-pi/4+0.01),0.02,0.02,'g');

plot(real(Gs4),imag(Gs4),'g.','MarkerSize',30);
th2(4) = text(0.05,0,'G(s_4) = 0.037');
arrowhead(Gs4*cos(pi/4),Gs4*sin(pi/4),...
    Gs4*cos(pi/4+0.01),Gs4*sin(pi/4+0.01),0.02,0.02,'g');

hold off;


%% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    set(th1,'FontSize',14)
    myprint(1,'ClosedCurveGam')
        
    figure(2)
    garyfyFigure        
    set(th2,'FontSize',14)
    myprint(2,'ClosedCurveGGam')
end
