%% Closed Curves: Perturbed by Epsilon
% This file generates plots to demonstrate the notation for a simple
% closed curve Gamma and its mapping G(Gamma).

%% Transfer Function
L = tf(1,[1 0]);

%% Closed Curves, Gamma_{R,epsilon}
% The curve Gamma goes from s=epsilon-jR to s=epsilon+jR along the 
% imaginary axis and then returns to s=epsilon-jR along a semicircle of 
% radius R in the RHP.

Npts = 500;

epsilon=-0.1;
R = 1.5;
Gamma1 = epsilon+j*linspace(-R,0,Npts);
Gamma2 = epsilon+j*linspace(0,R,Npts);

theta = linspace(pi/2,-pi/2,Npts);
Gamma3 = epsilon+R*exp(j*theta);

Gamma = [Gamma1(:); Gamma2(:); Gamma3(:)];

figure(1);
clf
plot(real(Gamma1),imag(Gamma1),'r:', real(Gamma2),imag(Gamma2),'b',...
        real(Gamma3), imag(Gamma3),'g-.');
xlabel('Real Part');
ylabel('Imag. Part');
axis equal
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Label specific points and plot arrowheads 
hold on;
th1 = [];

plot(epsilon,-R,'r.','MarkerSize',30);
th1(1) = text(-0.85,-0.92*R,'s_1 = \epsilon-jR');
arrowhead(epsilon,-R/2,epsilon,0,0.3,[],'r');

plot(epsilon,0,'b.','MarkerSize',30);
th1(2) = text(-0.65,0,'s_2 = \epsilon');
arrowhead(epsilon,R/2,epsilon,R,0.3,[],'b');

plot(epsilon,+R,'b.','MarkerSize',30);
th1(3) = text(-0.85,0.92*R,'s_3 = \epsilon+jR');
arrowhead(epsilon+R*cos(pi/4),R*sin(pi/4),...
   epsilon+R*cos(pi/4-0.01),R*sin(pi/4-0.01),0.3,[],'g');

plot(epsilon+R,0,'g.','MarkerSize',30);
th1(4) = text(1.6,0,'s_4 = \epsilon+R');
arrowhead(epsilon+R*cos(-pi/4),R*sin(-pi/4),...
   epsilon+R*cos(-pi/4-0.01),R*sin(-pi/4-0.01),0.3,[],'g');

hold off;

%% Mapping, L(Gamma)

L1 = freqresp(L,Gamma1);
L1 = L1(:);
L2 = freqresp(L,Gamma2);
L2 = L2(:);
L3 = freqresp(L,Gamma3);
L3 = L3(:);

figure(2);
clf
plot(real(L1),imag(L1),'r:', real(L2),imag(L2),'b',...
        real(L3), imag(L3),'g-.');
xlabel('Real Part');
ylabel('Imag. Part');
axis equal
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Label specific points and plot arrowheads 
hold on;
th2 = [];

Ls1 = evalfr(L,epsilon-j*R);
plot(real(Ls1),imag(Ls1),'r.','MarkerSize',30);
th2(1) = text(-5,1,'L(s_1) = -0.04+0.66j');
a1 = evalfr(L,epsilon-j*0.11);
a2 = evalfr(L,epsilon-j*0.11*0.99);
arrowhead(real(a1),imag(a1),real(a2),imag(a2),0.8,[],'r');

Ls2 = evalfr(L,epsilon);
plot(real(Ls2),imag(Ls2),'b.','MarkerSize',30);
th2(2) = text(-9.5,0,'L(s_2) = -10');
a1 = evalfr(L,epsilon+j*0.10);
a2 = evalfr(L,epsilon+j*0.10*1.01);
arrowhead(real(a1),imag(a1),real(a2),imag(a2),0.8,[],'b');

Ls3 = evalfr(L,epsilon+j*R);
Ls4 = evalfr(L,epsilon+R);
plot(real(Ls3),imag(Ls3),'b.','MarkerSize',30);
th2(3) = text(-5,-1,'L(s_3) = -0.04-0.66j');
a1 = evalfr(L,epsilon+R*exp(j*pi/3));
a2 = evalfr(L,epsilon+R*exp(j*pi/3*0.99));
arrowhead(real(a1),imag(a1),real(a2),imag(a2),0.6,[],'g');

plot(real(Ls4),imag(Ls4),'g.','MarkerSize',30);
th2(4) = text(1,0,'L(s_4) = 0.71');
a1 = evalfr(L,epsilon+R*exp(-j*pi/5));
a2 = evalfr(L,epsilon+R*exp(-j*pi/5*1.01));
arrowhead(real(a1),imag(a1),real(a2),imag(a2),0.6,[],'g');

xlim([-10.5 4])
hold off;

%% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    set(th1,'FontSize',14)
    myprint(1,'ClosedCurveEpsGam')
        
    figure(2)
    garyfyFigure        
    set(th2,'FontSize',12)
    myprint(2,'ClosedCurveEpsGGam')
end
