%% Nyquist Theorem Examples
% This file generates a Nyquist plots to demonstrate the use of the
% Nyquist stability theorem.


%% Nyquist Plots: Stable Systems
% L1 is stable but feedback system is unstable.
% L2 is stable and feedback system is stable.
L1 = tf(2,[1 4]);
L2 = tf([-2 2],[1 4]);

Nw = 1e3;
[L1re,L1im,w1] = nyquist(L1,logspace(-2,4,Nw));
L1re = L1re(:); L1im = L1im(:);

[L2re,L2im,w2] = nyquist(L2,logspace(-2,4,Nw));
L2re = L2re(:); L2im = L2im(:);

figure(1)
clf;
plot(L1re,L1im,'b',L2re,L2im,'r-.');
hold on;
plot(L1re,-L1im,'b',L2re,-L2im,'r-.');
hold off;
legend('L1','L2','Location','Best');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
[~,i] = min( abs(w1-4) );
h=arrowhead(L1re(i),L1im(i),L1re(i+1),L1im(i+1),0.2,[],'b');
h=arrowhead(L1re(i),-L1im(i),L1re(i-1),-L1im(i-1),0.2,[],'b');

[~,i] = min( abs(w2-4) );
h=arrowhead(L2re(i),L2im(i),L2re(i+1),L2im(i+1),0.2,[],'r');
h=arrowhead(L2re(i),-L2im(i),L2re(i-1),-L2im(i-1),0.2,[],'r');

%% Nyquist Plots: Unstable Systems
% L3 is unstable and feedback system is unstable.
% L2 is unstable but feedback system is stable.
L3 = tf(2,[1 -4]);
L4 = tf(8,[1 -4]);

[L3re,L3im,w3] = nyquist(L3,logspace(-2,4,Nw));
L3re = L3re(:); L3im = L3im(:);

[L4re,L4im,w4] = nyquist(L4,logspace(-2,4,Nw));
L4re = L4re(:); L4im = L4im(:);

figure(2)
clf;
plot(L3re,L3im,'b',L4re,L4im,'r-.');
hold on;
plot(L3re,-L3im,'b',L4re,-L4im,'r-.');
hold off;
legend('L3','L4','Location','Best');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
[~,i] = min( abs(w3-4) );
h=arrowhead(L3re(i),L3im(i),L3re(i+1),L3im(i+1),0.2,[],'b');
h=arrowhead(L3re(i),-L3im(i),L3re(i-1),-L3im(i-1),0.2,[],'b');

[~,i] = min( abs(w4-4) );
h=arrowhead(L4re(i),L4im(i),L4re(i+1),L4im(i+1),0.2,[],'r');
h=arrowhead(L4re(i),-L4im(i),L4re(i-1),-L4im(i-1),0.2,[],'r');

%% Nyquist Plots: Higher Order Systems
L5 = tf(2,[1 -1])*tf(100,[1 5 100]);

[L5re,L5im,w5] = nyquist(L5,logspace(-3,4,Nw));
L5re = L5re(:); L5im = L5im(:);

figure(3)
clf;
plot(L5re,L5im,'b',L5re,-L5im,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
[~,i] = min( abs(w5-1) );
h=arrowhead(L5re(i),L5im(i),L5re(i+1),L5im(i+1),0.2,[],'b');
h=arrowhead(L5re(i),-L5im(i),L5re(i-1),-L5im(i-1),0.2,[],'b');

%% Nyquist Plots: Higher Order Systems
L6 = tf(1,[1 1])^5;

[L6re,L6im,w6] = nyquist(L6,logspace(-3,4,Nw));
L6re = L6re(:); L6im = L6im(:);

figure(4)
clf;
plot(L6re,L6im,'b',L6re,-L6im,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
[~,i] = min( abs(w6-0.20) );
h=arrowhead(L6re(i),L6im(i),L6re(i+1),L6im(i+1),0.2,[],'b');
h=arrowhead(L6re(i),-L6im(i),L6re(i-1),-L6im(i-1),0.2,[],'b');


%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    axis equal
    myprint(1,'NyqThm1')
    
    figure(2)
    garyfyFigure
    axis equal
    myprint(2,'NyqThm2')
    
    figure(3)
    garyfyFigure
    axis equal
    myprint(3,'NyqThm3')

    figure(4)
    garyfyFigure
    axis equal
    myprint(4,'NyqThm4')
end
