%% Gain Margin Example
% This file computes the gain margin for a simple example.

%% Nominal design model and controller

% Design model
G = tf(1,[1 2 3 -1]);

% Proportional Controller
K = 2;

% Nominal Loop L(s)=G(s)*K(s)
L = G*K;

% Poles at g=1
% Note: G and K do not have a RHP pole/zero cancellation. Hence stability
% of the nominal closed-loop is equivalent to stability of the nominal 
% sensitivity function S(s)=1/(1+G(s)*K(s)). All poles of S(s) are in 
% the LHP so the nominal closed-loop is stable.
S = feedback(1,L);  
p1 = pole(S);
Npole = numel(p1);

%% Root Locus

% The allmargin command returns all gains g that result in closed-loop
% poles on the imaginary axis at s=+/-jw. The gains are stored in 
% AM.GainMargin and the frequencies w are stored in AM.GMFrequency.
AM = allmargin(L);
GM = AM.GainMargin;
GM_dB = 20*log10(GM);
wGM = AM.GMFrequency;

% Locus of closed-loop poles for 0<g<=1
% Note:  rlocus takes the loop L(s)=G(s)K(s) as an input and computes
% the closed-loop poles of S(s) = 1/(1+g*G(s)*K(s)) as g varies from 
% g==0 to g==infty.
glow = linspace(0,1,1e3);
plow = rlocus(L,glow);

% Locus of closed-loop poles for g>=1
gup = linspace(1,8,1e4);
pup = rlocus(L,gup);

% Closed-loop poles for specific values of g.
gval = [0 1 gup(end) GM];
pval = rlocus(L,gval);

% Plot poles as gain g varies above and below g=1
figure(1)
clf
for i=1:Npole
    ph1=plot(real(plow(i,:)),imag(plow(i,:)),'r-.');
    hold on;
    ph2=plot(real(pup(i,:)),imag(pup(i,:)),'b');
end
ph3 = [];
for i=1:numel(gval)
    ph3(i)=plot(real(pval(:,i)),imag(pval(:,i)),'kx');
end
xlabel('Real Part');
ylabel('Imag. Part');
legend([ph1 ph2],'g<1','g>1','Location','Best')
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
axis([-3 0.5 -2.5 2.5]);

%% Bode Plot of nominal loop L(s)=G(s)*K(s)
w = logspace(-2,1,100);
[Lmag,Lph_deg,w] = bode(L,w);
Lmag = Lmag(:);
Lph_deg = Lph_deg(:);
Lmag_dB = 20*log10(Lmag);

figure(2)
clf;

subplot(2,1,1)
semilogx(w,Lmag_dB,'b'); hold on;
semilogx([wGM(2) wGM(2) w(2)],[-40 -GM_dB(2) -GM_dB(2)],'k-.')
ylabel('Magnitude (dB)');
title(['Bode Diagram']);
grid on;
axis([w(1) w(end) -40 20]);
set(gca,'YTick',[-40 -20 -10.9 0 6 20],'LineWidth',1);

subplot(2,1,2)
semilogx(w,Lph_deg,'b'); hold on;
semilogx(w,Lph_deg,'b',[wGM(2) wGM(2) w(1)],[-270 -180 -180],'k-.')
axis([w(1) w(end) -270 0]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'YTick',[-270 -180 -90 0],'LineWidth',1);
grid on;



%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    set(ph3,'MarkerSize',12)
    myprint(1,'GMRootLocus2')
    
    figure(2)
    garyfyFigure
    myprint(2,'GMBode2')
    
%     figure(3)
%     garyfyFigure
%     %myprint(3,'GMNyquist1')
end
