%% Disk Margins on a Nyquist Plot
% This file illustrates the use of the Nyquist plot to determine
% disk margins.

%% Loop Transfer Function

% Design model
G = tf(1,[1 2 3 -1]);

% Proportional Controller
K = 2;

% Nominal Loop L(s)=G(s)*K(s)
L = G*K;

% Margins
AM = allmargin(L);

%% Nyquist Plot: Gain/Phase Margin Directions

% Generate Nyquist Curve
[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(1)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.15,[],'b');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.15,[],'b');
end

% Draw unit disk
theta = linspace(0,2*pi);
hold on;
ph2 = plot(cos(theta),sin(theta),'k:');

% Highlight gain/phase margin directions
g1 = AM.GainMargin(1);
g2 = AM.GainMargin(2);
ph3=plot([-1/g1 -1/g2],[0 0],'r');
arrowhead(-0.93/g1,0,-1/g1,0,0.15,[],'r');
arrowhead(-1.42/g2,0,-1/g2,0,0.15,[],'r');
th = AM.PhaseMargin*pi/180*linspace(-1,1,200);
ph4=plot( cos(-pi+th),sin(-pi+th),'r');
arrowhead(cos(-pi+th(20)),sin(-pi+th(20)),cos(-pi+th(1)),sin(-pi+th(1))...
    ,0.15,[],'r');
arrowhead(cos(-pi+th(end-20)),sin(-pi+th(end-20)),...
    cos(-pi+th(end)),sin(-pi+th(end)),0.15,[],'r');
arrowhead(-1.42/g2,0,-1/g2,0,0.15,[],'r');
ph5 = plot(-1,0,'g+','MarkerSize',18);
hold off;
axis equal
legend([ph1(1); ph2],'Nyquist for L','Unit Disk','Location','Southeast');

%% Nyquist Plot: Distance |1+L|

% Generate Nyquist Curve
[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(2)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.15,[],'b');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.15,[],'b');
end

% Draw vectors -1, L, and 1+L
hold on;
w = 0.35;
[Lre,Lim] = nyquist(L,w);
plot([0 -1],[0 0],'k:');
h=arrowhead(-0.85,0,-1,0,0.15,[],'k');
plot([0 Lre],[0 Lim],'k:');
h=arrowhead(0.9*Lre,0.9*Lim,Lre,Lim,0.15,[],'k');
plot([-1 Lre],[0 Lim],'r');
h=arrowhead(-0.2+0.8*Lre,0.8*Lim,Lre,Lim,0.15,[],'r');
plot(-1,0,'g+','MarkerSize',12);
hold off;
axis equal

th = [];
th(1) = text(-0.7,0.1,'-1');
th(2) = text(-0.4,-0.4,'L(j\omega)');
th(3) = text(-1.35,-0.3,'1+L(j\omega)');



%% Nyquist Plot: Disk Margins

% Generate Nyquist Curve
[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(3)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.15,[],'b');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.15,[],'b');
end

% Draw unit disk
theta = linspace(0,2*pi);
hold on;
ph2 = plot(cos(theta),sin(theta),'k:');

% Draw disk of radius r centered at critical -1 point
r = 0.4;
ph3 = patch(-1+r*cos(theta),r*sin(theta),'g','FaceAlpha',0.2);
ph4 = plot(-1,0,'g+','MarkerSize',18);

% Label Key points on Disk
plot(-1+r,0,'ko');
plot(-1-r,0,'ko');
a = 2*asin(r/2);
plot(-cos(a),sin(a),'ko');
plot(-cos(a),-sin(a),'ko');

th2 = []
th2(1) = text(-0.55,0,'-0.6');
th2(2) = text(-1.65,0,'-1.4');
th2(3) = text(-1.45,+0.5,'-0.92+0.39j');
th2(4) = text(-1.45,-0.5,'-0.92-0.39j');

hold off;
axis equal
%legend([ph1(1); ph2],'Nyquist for L','Unit Disk','Location','Southeast');



%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'GMPMDirections')        

    figure(2)
    garyfyFigure
    set(th,'FontSize',14);
    myprint(2,'Distance_1PlusL')     
    
    figure(3)
    garyfyFigure
    set(th2,'FontSize',12);
    myprint(3,'DiskMargin')        
end
