%% Gain/Phase Margins on a Nyquist Plot
% This file illustrates the use of the Nyquist plot to determine
% gain and phase margins.

%% Loop Transfer Function

% Design model
G = tf(1,[1 2 3 -1]);

% Proportional Controller
K = 2;

% Nominal Loop L(s)=G(s)*K(s)
L = G*K;

% Margins
AM = allmargin(L);

%% Nyquist Plot

% Generate Nyquist Curve
[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(1)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25 2.5];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.15,[],'b');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.15,[],'b');
end

% Draw unit disk
theta = linspace(0,2*pi);
hold on;
ph2 = plot(cos(theta),sin(theta),'k:');
ph3 = plot(-1,0,'g+','MarkerSize',18);
legend([ph1(1); ph2],'Nyquist for L','Unit Disk','Location','Southeast');
hold off;
axis equal

% Highlight data points at certain frequencies
w = [AM.GMFrequency -AM.PMFrequency AM.PMFrequency]; % Frequencies, rad/sec
Nw = numel(w);

[Lre,Lim,w] = nyquist(L,w);
Lre = Lre(:);
Lim = Lim(:);

hold on;
plot(Lre,Lim,'bo');
hold off;

th = [];
th(1) = text(Lre(1)+0.08,Lim(1),'L(0)');
th(2) = text(Lre(2)-0.5,Lim(2),'L(1.73j)');
th(3) = text(Lre(3)+0.1,Lim(3),'L(-0.49j)');
th(4) = text(Lre(4)+0.1,Lim(4),'L(0.49j)');

%% Nyquist Curve Scaled by Lower Gain Margin
g = AM.GainMargin(1);

[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(2)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
hold on;
ph2=plot(g*Lre,g*Lim,'r-.',g*Lre,-g*Lim,'r-.');
ph3=plot(cos(theta),sin(theta),'k:');
ph4 = plot(-1,0,'g+','MarkerSize',18);
hold off;
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
legend([ph1(1); ph2(1); ph3],'Nyquist for L','Nyquist for 0.5*L',...
    'Unit Disk','Location','Southeast');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.15,[],'b');
    h=arrowhead(g*Lre(1),g*Lim(1),g*Lre(2),g*Lim(2),0.15,[],'r');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.15,[],'b');
    h=arrowhead(g*Lre(1),-g*Lim(1),g*Lre(2),-g*Lim(2),0.15,[],'r');
end

%% Nyquist Curve Scaled by Upper Gain Margin
g = AM.GainMargin(2);

[Lre,Lim,ww] = nyquist(L);
Lre = Lre(:);
Lim = Lim(:);

figure(3)
clf;
ph1=plot(Lre,Lim,'b',Lre,-Lim,'b');
hold on;
ph2=plot(g*Lre,g*Lim,'r-.',g*Lre,-g*Lim,'r-.');
ph3=plot(cos(theta),sin(theta),'k:');
ph4 = plot(-1,0,'g+','MarkerSize',18);
hold off;
xlabel('Real Part');
ylabel('Imag. Part');
title('Nyquist Diagram');
legend([ph1(1); ph2(1); ph3],'Nyquist for L','Nyquist for 3.5*L',...
    'Unit Disk','Location','Southeast');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Plot arrowheads at certain frequencies
w = [0.25];
for i=1:numel(w)
    [Lre,Lim] = nyquist(L,[w(i) 1.1*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),Lim(1),Lre(2),Lim(2),0.4,[],'b');
    h=arrowhead(g*Lre(1),g*Lim(1),g*Lre(2),g*Lim(2),0.4,[],'r');
    
    
    [Lre,Lim] = nyquist(L,[w(i) 0.9*w(i)]);
    Lre = Lre(:);
    Lim = Lim(:);
    h=arrowhead(Lre(1),-Lim(1),Lre(2),-Lim(2),0.4,[],'b');
    h=arrowhead(g*Lre(1),-g*Lim(1),g*Lre(2),-g*Lim(2),0.4,[],'r');
end


%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    set(th,'FontSize',14);
    myprint(1,'GMPMNyquist')    
    
    figure(2)
    garyfyFigure
    myprint(2,'LowerGMNyquist')    
    
    figure(3)
    garyfyFigure
    myprint(3,'UpperGMNyquist')    
end
